package com.ec.pf.screenlet.common.model.ejb;

import java.sql.Timestamp;
import java.text.ParseException;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import javax.annotation.PreDestroy;
import javax.annotation.Resource;
import javax.annotation.security.DeclareRoles;
import javax.annotation.security.RolesAllowed;
import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.FinderException;
import javax.ejb.Init;
import javax.ejb.Local;
import javax.ejb.LocalHome;
import javax.ejb.ObjectNotFoundException;
import javax.ejb.Remove;
import javax.ejb.SessionContext;
import javax.ejb.Stateful;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import org.apache.log4j.Logger;

import com.ec.eccore.util.date.DateHelper;
import com.ec.eccore.util.session.ECSessionContext;
import com.ec.eccore.util.sql.ConnectionFactory;
import com.ec.pf.screenlet.common.model.entities.SystMessAcknowledgeEntity;
import com.ec.pf.screenlet.common.model.entities.SystMessRoleEntity;
import com.ec.pf.screenlet.common.model.entities.SystemMessageEntity;

@Stateful
@Local(SystemMessageLocal.class)
@LocalHome(SystemMessageEJBHome.class)
@TransactionAttribute(TransactionAttributeType.REQUIRES_NEW)
@DeclareRoles({"WEB"})
@RolesAllowed({"WEB"})
public class SystemMessageEJB{
	Logger cat = Logger.getLogger(this.getClass());

	@Resource
	private SessionContext ctx;

	@PersistenceContext(unitName = "eccore-ejb")
	EntityManager entityManager;

	public String message = null;
	public String subject = null;
	public String expireDate = null;
	public String postDate = null;
	public String lastUpdatedDate = null;
	public String messageType = null;
	public String priority = null;
	public Integer messageNo;
	public HashMap ackUsers;
	public HashMap roles;
	private boolean messageDirty;
	private boolean ackUserDirty;
	private boolean rolesDirty;
	private String dirty = new String("dirty");
	private String clean = new String("clean");
	private String sql;
	private SystemMessagePK primaryKey = null;

	private final static String DBDATEFORMAT = "'YYYY-MM-DD\"T\"HH24:MI:SS'";
	private final static String TODATE = "to_date(?, " + DBDATEFORMAT + ")";

	private static String hintName = "org.hibernate.cacheable";

	@Init
	public SystemMessagePK init(String msg, String sbj, String expDate,
			String pDate) throws CreateException, ParseException {
		return init(msg, sbj, "", "", expDate, pDate);
	}

	@Init
	private SystemMessagePK init(String msg, String sbj, String msg_type,
			String priority, String expDate, String pDate)
			throws CreateException, ParseException {
		cat.debug("About to create new SystemMessage entity");
		messageDirty = false;
		ackUserDirty = false;
		rolesDirty = false;
		setMessage(msg);
		setSubject(sbj);
		setMsgType(msg_type);
		setPriority(priority);
		setLastUpdatedDate(DateHelper.SystemNow());
		ackUsers = new HashMap();
		roles = new HashMap();
		sql = "insert into system_message (message_no, message, message_subject, message_type, priority, expire_date, post_date, last_updated_date ) values (?,?,?,?,?,"
				+ TODATE + "," + TODATE + "," + TODATE + ")";

		if (expDate != "") {
			this.setExpireDate(expDate);
		}
		if (pDate != "") {
			this.setPostDate(pDate);
		}

		messageNo = new Integer(getNewObjectId());

		SystemMessageEntity systemMessageEntity = new SystemMessageEntity();
		systemMessageEntity.setMessage_no(messageNo.longValue());
		systemMessageEntity.setMessage(message);
		systemMessageEntity.setMessage_subject(subject);
		systemMessageEntity.setMessage_type(messageType);
		systemMessageEntity.setPriority(priority);
		if (expireDate != null) {
			Timestamp date = Timestamp.valueOf(expireDate);
			systemMessageEntity.setExpire_date(date);
		}
		if (postDate != null) {
			Timestamp date = Timestamp.valueOf(postDate);
			systemMessageEntity.setPost_date(date);
		}

		if (lastUpdatedDate != null) {
			Timestamp date = Timestamp.valueOf(lastUpdatedDate);
			systemMessageEntity.setLast_updated_date(date);
		}

		if (cat.isDebugEnabled()) {
			cat.debug("About to execute:\n\t" + sql + "\nwith parameters:\n\t"
					+ messageNo.toString() + "\n\t" + message + "\n\t"
					+ subject + "\n\t" + messageType + "\n\t" + priority
					+ "\n\t" + expireDate + "\n\t" + postDate + "\n\t"
					+ lastUpdatedDate);
		}
		entityManager.persist(systemMessageEntity);

		cat.debug("Statement executed");

		if (cat.isDebugEnabled()) {
			cat.debug("ejbCreate for message with id " + this.getMessageNo()
					+ " termitating..");
		}
		return new SystemMessagePK(messageNo,
				ConnectionFactory.getConfigKey(ctx.getCallerPrincipal()));
	}

	private int getNewObjectId() {
		int assignId;

		String sql = "call EcDp_System_Key.assignNextNumber(?, ?)";

		Query query = entityManager.createNativeQuery(sql);
		query.setParameter(1, "SYSTEM_MESSAGE");
		assignId = (Integer) query.getSingleResult();

		return assignId;
	}

	@PreDestroy
	public void deleteSystemMessage() throws EJBException {
		if (cat.isDebugEnabled()) {
			cat.debug("About to remove data for SystemMessage entity with message_no: "
					+ this.getMessageNo());
		}
		if (this.ackUsers.size() > 0) {
			sql = "delete from syst_mess_acknowledge where message_no = ?"; // Just
																			// for
																			// debugging
																			// purpose
			SystMessAcknowledgeEntity systMessAcknowledgeEntity = new SystMessAcknowledgeEntity();
			systMessAcknowledgeEntity.setMessage_no(messageNo.longValue());
			if (cat.isDebugEnabled()) {
				cat.debug("About to remove acknowledged users width the following statement:\n\t"
						+ sql + "\nwith parameters:\n\t" + this.getMessageNo());
			}
			try {
				entityManager.remove(entityManager
						.merge(systMessAcknowledgeEntity));
			} catch (Exception e) {
				throw new EJBException(
						"Failed to remove ack users for message "
								+ this.getMessageNo());
			}
			cat.debug("Delete statement executed");
		} else {
			if (cat.isDebugEnabled()) {
				cat.debug("Message "
						+ this.getMessageNo()
						+ " did not have any acknowledged users, uneccesary to delete in db");
			}
		}
		sql = "delete from system_message where message_no = ?";
		SystemMessageEntity systemMessageEntity = new SystemMessageEntity();
		systemMessageEntity.setMessage_no(messageNo.longValue());
		if (cat.isDebugEnabled()) {
			cat.debug("About to remove system message width the following statement:\n\t"
					+ sql + "\nwith parameters:\n\t" + this.getMessageNo());
		}
		try {
			entityManager.remove(entityManager.merge(systemMessageEntity));
		} catch (Exception e) {
			throw new EJBException("Failed to remove message "
					+ this.getMessageNo());
		}
		cat.debug("Delete statement executed");
	}

	// Called after a business method with a transaction is called. The
	// container will call the method for us
	public void ejbStore() throws EJBException {
		if (cat.isDebugEnabled()) {
			cat.debug("About to store data for SystemMessage entity with message_no: "
					+ messageNo.toString());
		}
		if (!messageDirty && !ackUserDirty && !rolesDirty) {
			cat.debug("No dirty data, no need to store");
		}
		if (messageDirty) {
			sql = ("update system_message set message = ?, message_subject = ?, message_type = ?, priority = ?, expire_date = "
					+ TODATE
					+ " , post_date = "
					+ TODATE
					+ ", last_updated_date = " + TODATE + " where message_no = ?");
			SystemMessageEntity systemMessageEntity = new SystemMessageEntity();
			systemMessageEntity.setMessage(this.getMessage());
			systemMessageEntity.setMessage_subject(this.getSubject());
			systemMessageEntity.setMessage_type(this.getMsgType());
			systemMessageEntity.setPriority(this.getPriority());
			String expiryDate = this.getExpireDate();
			if (expiryDate != null) {
				Timestamp date = Timestamp.valueOf(expiryDate);
				systemMessageEntity.setExpire_date(date);
			}
			String postDate = this.getPostDate();
			if (postDate != null) {
				Timestamp date = Timestamp.valueOf(postDate);
				systemMessageEntity.setPost_date(date);
			}
			String lastUpdateDate = this.getLastUpdatedDate();
			if (lastUpdateDate != null) {
				Timestamp date = Timestamp.valueOf(lastUpdateDate);
				systemMessageEntity.setLast_updated_date(date);
			}
			systemMessageEntity.setMessage_no(messageNo.longValue());
			if (cat.isDebugEnabled()) {
				cat.debug("Message is dirty, about to update with the following statement:\n\t"
						+ sql
						+ "\nwith parameters:\n\t"
						+ this.getMessage()
						+ "\n\t"
						+ this.getSubject()
						+ "\n\t"
						+ this.getMsgType()
						+ "\n\t"
						+ this.getPriority()
						+ "\n\t"
						+ this.getExpireDate()
						+ "\n\t"
						+ this.getPostDate()
						+ "\n\t"
						+ this.getLastUpdatedDate()
						+ "\n\t"
						+ this.getMessageNo());
			}
			try {
				entityManager.merge(systemMessageEntity);
			} catch (Exception e) {
				throw new EJBException("Failed to save object state");
			}
			cat.debug("Clearing the ackusers ejb buffer");
			ackUsers.clear();
			cat.debug("Statement executed...have to delete all acknowledged users for message");
			// Deleting all acknowledged users for message, because the message
			// have been updated
			sql = "delete from syst_mess_acknowledge where message_no = ?";
			SystMessAcknowledgeEntity systMessAcknowledgeEntity = new SystMessAcknowledgeEntity();
			systMessAcknowledgeEntity.setMessage_no(messageNo.longValue());
			if (cat.isDebugEnabled()) {
				cat.debug("About to delete with the following statement:\n\t"
						+ sql + "\nwith parameters:\n\t" + this.getMessageNo());
			}
			try {
				entityManager.remove(entityManager
						.merge(systMessAcknowledgeEntity));
			} catch (Exception e) {
				if (cat.isDebugEnabled()) {
					cat.debug("No users have acknowledged message "
							+ messageNo.intValue()
							+ "therefore no records are deleted");
				}
			}
			if (cat.isDebugEnabled()) {
				cat.debug("Statement executed...");
			}
			messageDirty = false;
		}

		if (ackUserDirty) {
			cat.debug("Ackusers is dirty, have to iterate through ackUsers hashMap and insert record in syst_mess_acknowledge for all users that have have mapped value = dirty");
			Iterator iter = ackUsers.keySet().iterator();
			while (iter.hasNext()) {
				String key = new String((String) iter.next());
				String value = new String((String) ackUsers.get(key));
				if (value.equals(dirty)) {
					sql = "insert into syst_mess_acknowledge(message_no, user_id) values (?,?)";
					SystMessAcknowledgeEntity entity = new SystMessAcknowledgeEntity();
					entity.setMessage_no(messageNo.longValue());
					entity.setUser_id(key.toString());

					if (cat.isDebugEnabled()) {
						cat.debug("User "
								+ key.toString()
								+ " has acknowledged but is "
								+ value.toString()
								+ " the following insert statement will be executed:");
						cat.debug("\n\t" + sql + "\nwith parameters:\n\t"
								+ this.getMessageNo() + "\n\t" + key.toString());
					}
					try {
						entityManager.persist(entity);
					} catch (Exception e) {
						throw new EJBException(
								"Failed to insert acknowledged users");
					}
					ackUsers.put(key, clean);
					cat.debug("Insert statement executed...");
				}
			}
		}

		if (ackUserDirty) {
			cat.debug("Ackusers is dirty, have to iterate through ackUsers hashMap and insert record in syst_mess_acknowledge for all users that have have mapped value = dirty");
			Iterator iter = ackUsers.keySet().iterator();
			while (iter.hasNext()) {
				String key = new String((String) iter.next());
				String value = new String((String) ackUsers.get(key));
				if (value.equals(dirty)) {
					sql = "insert into syst_mess_acknowledge(message_no, user_id) values (?,?)";

					SystMessAcknowledgeEntity entity = new SystMessAcknowledgeEntity();
					entity.setMessage_no(messageNo.longValue());
					entity.setUser_id(key.toString());

					if (cat.isDebugEnabled()) {
						cat.debug("User "
								+ key.toString()
								+ " has acknowledged but is "
								+ value.toString()
								+ " the following insert statement will be executed:");
						cat.debug("\n\t" + sql + "\nwith parameters:\n\t"
								+ this.getMessageNo() + "\n\t" + key.toString());
					}
					try {
						entityManager.persist(entity);
					} catch (Exception e) {
						throw new EJBException(
								"Failed insert acknowledged users");
					}
					ackUsers.put(key, clean);
					cat.debug("Insert statement executed...");
				}
			}
			if (!ackUsers.containsValue(dirty)) {
				ackUserDirty = false;
			} else {
				cat.debug("Error ackuser hashmap should not contain dirty users after db store");
			}
		}

		if (rolesDirty) {
			sql = "delete from syst_mess_role where message_no = ?";
			SystMessRoleEntity entity = new SystMessRoleEntity();
			entity.setMessage_no(messageNo.longValue());

			if (cat.isDebugEnabled()) {
				cat.debug("Roles is dirty have to delete all roles for message with the following statement:\n\t"
						+ sql + "\nwith parameters:\n\t" + this.getMessageNo());
			}
			try {
				entityManager.remove(entityManager.merge(entity));
			} catch (Exception e) {
				cat.debug("No roles associated with message "
						+ messageNo.intValue()
						+ "therefore no records are deleted");
			}
			if (cat.isDebugEnabled()) {
				cat.debug("Statement successfully executed");
			}

			// //Iterate through roles. insert record in syst_mess_role for all
			// roles
			Iterator iter = roles.keySet().iterator();
			while (iter.hasNext()) {
				Object key = iter.next();
				String roleAppId = (String) key;
				if (cat.isDebugEnabled()) {
					cat.debug("Traversing roles hashmap, current roleId is "
							+ roleAppId);
				}
				String roleId = roleAppId.substring(0,
						roleAppId.indexOf("@appl"));
				String appId = roleAppId
						.substring(roleAppId.indexOf("@appl") + 5);
				String roleName = (String) roles.get(key);
				sql = "insert into syst_mess_role(message_no, role_id, app_id) values (?,?,?)";
				SystMessRoleEntity systMessRoleEntity = new SystMessRoleEntity();
				systMessRoleEntity.setMessage_no(messageNo.longValue());
				systMessRoleEntity.setRole_id(roleId);
				systMessRoleEntity.setApp_id(new Long(appId));
				if (cat.isDebugEnabled()) {
					cat.debug("Inserting role with the following statement:\n\t"
							+ sql
							+ "\nwith parameters:\n\t"
							+ this.getMessageNo()
							+ "\n\t"
							+ roleId
							+ "\n\t"
							+ appId);
				}
				try {
					entityManager.persist(systMessRoleEntity);
				} catch (Exception e) {
					throw new EJBException("Failed to insert roles in db");
				}
				roles.put(key, roleName);
				cat.debug("Insert statement executed");
			}
			rolesDirty = false;
		}

		if (cat.isDebugEnabled()) {
			cat.debug("ejbStore for message with id " + this.getMessageNo()
					+ " termitating..");
		}
	}

	/**
	 * N.B:- ejbLoad is called before a business method wich is associated with
	 * a transaction is called. Thus this method acts as an ejbLoad. Before any
	 * business methods are called on the ejb, we have to explicitly call this
	 * method, since the container wouldn't do it for us as it has been renamed.
	 * I implemented own method instead of using ejbLoad because of
	 * EntityContext.getPrimaryKey() to get Primary key which is not implemented
	 * in EJB3. Thus we have to pass the primary key as a parameter to the
	 * method. And also ejbLoad doesn't take any parameters.
	 **/
	public void loadEJB(SystemMessagePK pk) throws EJBException {
		primaryKey = pk;
		if (cat.isDebugEnabled()) {
			cat.debug("About to load data for system message entity with PK:"
					+ primaryKey.getMessageNo());
		}
		// Updateing the message fields
		sql = "select message_no, message, message_subject, message_type, priority, to_char(last_updated_date,"
				+ DBDATEFORMAT
				+ "), to_char(expire_date,"
				+ DBDATEFORMAT
				+ "), to_char(post_date,"
				+ DBDATEFORMAT
				+ ") from system_message where message_no = ? order by message_no";
		SystemMessageEntity entity;
		if (cat.isDebugEnabled()) {
			cat.debug("Updating the message fields, About to execute:\n\t"
					+ sql + "\nwith parameters:\n\t"
					+ primaryKey.getMessageNo());
		}

		entity = entityManager.find(SystemMessageEntity.class, primaryKey
				.getMessageNo().longValue());
		cat.debug("Select statement executed");
		if (entity == null) {
			throw new EJBException("Object not found");
		}
		messageNo = entity.getMessage_no().intValue();
		this.setMessage(entity.getMessage());
		this.setSubject(entity.getMessage_subject());
		this.setMsgType(entity.getMessage_type());
		this.setPriority(entity.getPriority());
		Timestamp lastDate = entity.getLast_updated_date();
		if (lastDate != null) {
			this.setLastUpdatedDate(lastDate.toString());
		}
		Timestamp expiryDate = entity.getExpire_date();
		if (expiryDate != null) {
			this.setExpireDate(expiryDate.toString());
		}
		Timestamp postDateTimeStamp = entity.getPost_date();
		if (postDateTimeStamp != null) {
			this.setPostDate(postDateTimeStamp.toString());
		}
		messageDirty = false;

		cat.debug("Updating the ackUsers HashMap");
		sql = "select sma.user_id from SystMessAcknowledgeEntity sma where sma.message_no = :messageNumber";
		Query query = entityManager.createQuery(sql);
		query.setParameter("messageNumber", primaryKey.getMessageNo()
				.longValue());
		query.setHint(hintName, true);

		if (cat.isDebugEnabled()) {
			cat.debug("Updating the ackUser hashMap, About to execute:\n\t"
					+ sql + "\nwith parameters:\n\t" + messageNo.toString());
		}
		List<String> userIds = query.getResultList();
		cat.debug("Query executed, and has returned the following users:");
		ackUsers = new HashMap();
		int size = userIds.size();
		String userId = null;
		if (size != 0) {
			for (int i = 0; i < size; i++) {
				userId = userIds.get(i);
				String currUser = new String(userId);
				cat.debug(currUser);
				ackUsers.put(currUser, clean);
			}
		}

		ackUserDirty = false;

		cat.debug("Updating the roles HashMap");
		sql = "select smr.role_id, smr.app_id, tbr.role_name from SystMessRoleEntity smr, BasisRoleEntity tbr where smr.message_no = :messageNumber and smr.role_id = tbr.role_id";
		Query queryRoles = entityManager.createQuery(sql);
		queryRoles.setParameter("messageNumber", primaryKey.getMessageNo()
				.longValue());
		queryRoles.setHint(hintName, true);

		if (cat.isDebugEnabled()) {
			cat.debug("Updating the roles hashMap, About to execute:\n\t" + sql
					+ "\nwith parameters:\n\t" + messageNo.toString());
		}
		List<Object> results = queryRoles.getResultList();
		cat.debug("Query executed, and has returned the following roles:");
		roles = new HashMap();
		int resultSize = results.size();
		if (resultSize != 0) {
			for (int i = 0; i < resultSize; i++) {
				Object[] result = (Object[]) results.get(i);
				String currRoleId = new String((String) result[0]);
				Long appId = (Long) result[1];
				String currAppId = appId.toString();
				String currRoleName = new String((String) result[2]);
				roles.put(currRoleId.concat("@appl").concat(currAppId),
						currRoleName);
			}
		}
		rolesDirty = false;
		if (cat.isDebugEnabled()) {
			cat.debug("ejbLoad for message with id " + this.getMessageNo()
					+ " termitating..");
		}
	}

	public String getMessage() {
		return message;
	}

	public String getMessageNo() {
		return messageNo.toString();
	}

	public String getSubject() {
		return subject;
	}

	public String getLastUpdatedDate() {
		return lastUpdatedDate;
	}

	public String getExpireDate() {
		return expireDate;
	}

	public String getPostDate() {
		return postDate;
	}

	public HashMap getRoles() {
		return roles;
	}

	public String getAckCount() {
		String returnvalue = "";
		if (this.ackUsers != null) {
			returnvalue = Integer.toString(this.ackUsers.size());
		}
		return returnvalue;
	}

	public String getMsgType() {
		return messageType;
	}

	public String getPriority() {
		return priority;
	}

	public void setMessage(String message) {
		lastUpdatedDate = DateHelper.SystemNow();
		this.message = message;
		messageDirty = true;
	}

	public void setSubject(String subject) {
		lastUpdatedDate = DateHelper.SystemNow();
		this.subject = subject;
		messageDirty = true;
	}

	public void setMsgType(String msgType) {
		lastUpdatedDate = DateHelper.SystemNow();
		this.messageType = msgType;
		messageDirty = true;
	}

	public void setPriority(String priority) {
		lastUpdatedDate = DateHelper.SystemNow();
		this.priority = priority;
		messageDirty = true;
	}

	public void setLastUpdatedDate(String mD) {
		this.lastUpdatedDate = mD;
		if (this.lastUpdatedDate != "") {
			this.lastUpdatedDate = this.lastUpdatedDate.replace(' ', 'T');
		}
		messageDirty = true;
	}

	public void setExpireDate(String eD) {
		lastUpdatedDate = DateHelper.SystemNow();
		this.expireDate = eD;
		if (this.expireDate != "" && this.expireDate != null) {
			this.expireDate = this.expireDate.replace(' ', 'T');
		}
		// messageDirty = true; Removed because message should not be rposted
		// after modification of the expiredate
	}

	public void setPostDate(String pD) {
		lastUpdatedDate = DateHelper.SystemNow();
		this.postDate = pD;
		if (this.postDate != "" && this.postDate != null) {
			this.postDate = this.postDate.replace(' ', 'T');
		}
		messageDirty = true;
	}

	public void acknowledge() {
		String user = null;
		java.security.Principal principal = ctx.getCallerPrincipal();
		if (principal != null) {
			user = new String(ECSessionContext.getUserId(principal.getName()));
		}
		if (cat.isDebugEnabled()) {
			cat.debug("Acknowledge called for message" + messageNo
					+ " and user " + user);
		}
		if (!ackUsers.containsKey(user)) {
			ackUsers.put(user, dirty);
			if (cat.isDebugEnabled()) {
				cat.debug("Added user " + user
						+ " to ackUsers property for message " + getMessageNo());
			}
			ackUserDirty = true;
		} else {
			if (cat.isDebugEnabled()) {
				cat.debug("ackUsers already contained user " + user
						+ " for message " + getMessageNo());
			}
		}
	}

	public void addRole(String roleAppId, String roleName) {
		if (!roles.containsKey(roleAppId)) {
			roles.put(roleAppId, roleName);
			if (cat.isDebugEnabled()) {
				cat.debug("Added role application data " + roleAppId
						+ " to roles property for message " + getMessageNo());
			}
			rolesDirty = true;
		} else {
			if (cat.isDebugEnabled()) {
				cat.debug("roles already contained role " + roleAppId
						+ " for message " + getMessageNo());
			}
		}
	}

	public void removeRole(String roleAppId) {
		if (roles.containsKey(roleAppId)) {
			roles.remove(roleAppId);
			if (cat.isDebugEnabled()) {
				cat.debug("Removed " + roleAppId
						+ " from roles property for message " + getMessageNo());
			}
			rolesDirty = true;
		} else {
			if (cat.isDebugEnabled()) {
				cat.debug("Role " + roleAppId
						+ " is not associated with the message "
						+ getMessageNo() + " could not remove it");
			}
		}
	}

	public void updateRole(String fromRoleAppId, String toRoleAppId,
			String toRoleName) {
		this.removeRole(fromRoleAppId);
		this.addRole(toRoleAppId, toRoleName);
	}

	public SystemMessagePK ejbFindByPrimaryKey(SystemMessagePK pk)
			throws FinderException {
		if (cat.isDebugEnabled()) {
			cat.debug("Attempting to find message with id: "
					+ Integer.toString(pk.getMessageNo().intValue()));
		}
		SystemMessageEntity systemMessageEntity = entityManager.find(
				SystemMessageEntity.class, pk.getMessageNo().longValue());
		if (systemMessageEntity == null) {
			throw new ObjectNotFoundException();
		}
		return pk;
	}
	
	public Collection<SystemMessagePK> ejbFindMessages() throws FinderException {
		cat.debug("Finding messages...");

		LinkedList<SystemMessagePK> queryMatches = new LinkedList<SystemMessagePK>();
		try {
			sql = "select message_no from SystemMessageEntity order by message_no desc";
			Query findMessagesQuery = entityManager.createQuery(sql);
			findMessagesQuery.setHint(hintName, true);
			cat.debug(
				"About to find all messages with the following query:\n\t" + sql);
			List <Long> findMessagesResultList = findMessagesQuery.getResultList(); 
			cat.debug("Query executed...");
			int size = findMessagesResultList.size();
			Integer messageNumber;
			if(size!=0){
				for(int i=0; i<size; i++){
					messageNumber = findMessagesResultList.get(i).intValue();
					SystemMessagePK pk = new SystemMessagePK(messageNumber, ConnectionFactory.getConfigKey(ctx.getCallerPrincipal()));
					queryMatches.add(pk);
				}
			}
		} catch (Exception e) {
			throw new FinderException(e.getMessage());
		} 
		return queryMatches;
	}
	
	public Collection<SystemMessagePK> ejbFindMessages(String startDate, String endDate)
			throws FinderException {
			LinkedList<SystemMessagePK> queryMatches = new LinkedList<SystemMessagePK>();
			try {
				sql = "select message_no from SystemMessageEntity where last_updated_date >= to_char(:startDate, "+DBDATEFORMAT+") and last_updated_date <= to_char(:endDate, "+DBDATEFORMAT+") order by message_no desc";
				Query findQuery = entityManager.createQuery(sql);
				findQuery.setParameter("startDate", true);
				findQuery.setParameter("endDate", true);
				findQuery.setHint(hintName, true);
				
				if (cat.isDebugEnabled()) {
					cat.debug(
						"About to find messages last modified between "
							+ startDate
							+ " and "
							+ endDate
							+ "with the following quert\n\t"
							+ sql
							+ "with parameters\n\t"
							+ startDate
							+ "\n\t"
							+ endDate);
				}
				List<Long> findResusltList = findQuery.getResultList();
				cat.debug("Query executed");
				int size = findResusltList.size();
				if(size!=0){
					for(int i =0; i<size; i++){
						Integer result = findResusltList.get(i).intValue();
						SystemMessagePK pk =
								new SystemMessagePK(
									result,
									ConnectionFactory.getConfigKey(ctx.getCallerPrincipal()));
							queryMatches.add(pk);
								}
				}
				
			} catch (Exception e) {
				throw new FinderException(e.getMessage());
			} 
			return queryMatches;
		}

	public Collection<SystemMessagePK> ejbFindUnAckMessages() throws FinderException {
		String now = null;
		cat.debug("ejbFindUnAckMessages starting...");
		String user = null;
		java.security.Principal principal = ctx.getCallerPrincipal();
		if (principal != null) {
			user = new String(ECSessionContext.getUserId(principal.getName()));
			cat.debug("About to find unacknowledged messages for user " + user);
		}
		LinkedList<SystemMessagePK> queryMatches = new LinkedList<SystemMessagePK>();
		try {
			now = DateHelper.SystemNow();
			sql =
					"select message_no from SystemMessageEntity sm where sm.message_no in(select message_no from (select message_no from SystMessRoleEntity where role_id in (select role_id from BasisUserRoleEntity t where t.user_id  like :tUserId)) t where t.message_no not in(select message_no from SystMessAcknowledgeEntity u where u.user_id like :uUserId)) and nvl(sm.expire_date,"
						+ "to_char(:now1, "+DBDATEFORMAT+")"
						+ ") >= "
						+ "to_char(:now2, "+DBDATEFORMAT+")"
						+ " and nvl(sm.post_date,"
						+ "to_char(:now3, "+DBDATEFORMAT+")"
						+ ") <= "
						+ "to_char(:now4, "+DBDATEFORMAT+")"
						+ " and NVL(sm.message_type,'SYSTEM_MESSAGE')='SYSTEM_MESSAGE'"
						+ " order by message_no desc";
			
			Query selectQuery = entityManager.createQuery(sql);
			selectQuery.setParameter("tUserId", user);
			selectQuery.setParameter("uUserId", user);
			selectQuery.setParameter("now1", now);
			selectQuery.setParameter("now2", now);
			selectQuery.setParameter("now3", now);
			selectQuery.setParameter("now4", now);
			if (cat.isDebugEnabled()) {
				cat.debug(
					"About to execute:\n\t"
						+ sql
						+ "\nwith parameters:\n\t"
						+ user
						+ "\n\t"
						+ user
						+ "\n\t"
						+ now
						+ "\n\t"
						+ now);
			}
			selectQuery.setHint(hintName, true);
			List<Long> resultList = selectQuery.getResultList();
			
			cat.debug("Query executed...");
			
			int size = resultList.size();
			if(size!=0){
				for(int i = 0; i<size; i++){
					Integer result = resultList.get(i).intValue();
					SystemMessagePK pk =
							new SystemMessagePK(
								result,
								ConnectionFactory.getConfigKey(ctx.getCallerPrincipal()));
						queryMatches.add(pk);
				}
			}
		} catch (Exception e) {
			cat.fatal("Could not find system_message table");
		} 
		cat.debug("ejbFindUnAckMessages finishing...");
		return queryMatches;
	}
	

}